"""Base class for all objects requiring a valid names for all engines."""
from pydantic import BaseModel, Field, Extra


class NoExtraBaseModel(BaseModel):
    """Base class for all objects that are not extensible with additional keys.
    
    This effectively includes all objects except for the Properties classes
    that are assigned to geometry objects.
    """

    class Config:
        extra = Extra.forbid


class IDdBaseModel(NoExtraBaseModel):
    """Base class for all objects requiring a identifiers acceptable for all engines."""

    identifier: str = Field(
        ...,
        regex=r'[A-Za-z0-9_-]',
        min_length=1,
        max_length=100,
        description='Text string for a unique object ID. This identifier remains '
            'constant as the object is mutated, copied, and serialized to different '
            'formats (eg. dict, idf, rad). This identifier is also used to reference '
            'the object across a Model. It must be < 100 characters and not contain '
            'any spaces or special characters.'
    )

    display_name: str = Field(
        default=None,
        description='Display name of the object with no character restrictions.'
    )

    user_data: dict = Field(
        default=None,
        description='Optional dictionary of user data associated with the object.'
            'All keys and values of this dictionary should be of a standard data '
            'type to ensure correct serialization of the object (eg. str, float, '
            'int, list).'
    )



from honeybee_schema.energy.hvac import IdealAirSystemAbridged
import os

# target folder where all of the samples live
root = os.path.dirname(os.path.dirname(__file__))
target_folder = os.path.join(root, 'samples', 'hvac')


def test_ideal_air_default():
    file_path = os.path.join(target_folder, 'ideal_air_default.json')
    IdealAirSystemAbridged.parse_file(file_path)


def test_ideal_air_detailed():
    file_path = os.path.join(target_folder, 'ideal_air_detailed.json')
    IdealAirSystemAbridged.parse_file(file_path)



"""ConstructionSet Schema"""
from pydantic import Field, constr
from typing import List, Union

from .._base import NoExtraBaseModel
from ._base import IDdEnergyBaseModel
from .construction import OpaqueConstructionAbridged, WindowConstructionAbridged, \
    ShadeConstruction, AirBoundaryConstructionAbridged, OpaqueConstruction, \
    WindowConstruction, AirBoundaryConstruction
from .material import EnergyMaterial, EnergyMaterialNoMass, \
    EnergyWindowMaterialGas, EnergyWindowMaterialGasCustom, \
    EnergyWindowMaterialGasMixture, EnergyWindowMaterialSimpleGlazSys, \
    EnergyWindowMaterialBlind, EnergyWindowMaterialGlazing, EnergyWindowMaterialShade


class _FaceSubSetAbridged(NoExtraBaseModel):
    """A set of constructions for wall, floor, or roof assemblies."""

    interior_construction: str = Field(
        default=None,
        min_length=1,
        max_length=100,
        description='Identifier for an OpaqueConstruction for faces with a Surface or '
            'Adiabatic boundary condition.'
    )

    exterior_construction: str = Field(
        default=None,
        min_length=1,
        max_length=100,
        description='Identifier for an OpaqueConstruction for faces with an Outdoors '
            'boundary condition.'
    )

    ground_construction: str = Field(
        default=None,
        min_length=1,
        max_length=100,
        description='Identifier for an OpaqueConstruction for faces with a Ground '
            'boundary condition.'
    )


class WallConstructionSetAbridged(_FaceSubSetAbridged):
    """A set of constructions for wall assemblies."""

    type: constr(regex='^WallConstructionSetAbridged$') = \
        'WallConstructionSetAbridged'


class FloorConstructionSetAbridged(_FaceSubSetAbridged):
    """A set of constructions for floor assemblies."""

    type: constr(regex='^FloorConstructionSetAbridged$') = \
        'FloorConstructionSetAbridged'


class RoofCeilingConstructionSetAbridged(_FaceSubSetAbridged):
    """A set of constructions for roof and ceiling assemblies."""

    type: constr(regex='^RoofCeilingConstructionSetAbridged$') = \
        'RoofCeilingConstructionSetAbridged'


class _FaceSubSet(NoExtraBaseModel):
    """A set of constructions for wall, floor, or roof assemblies."""

    interior_construction: OpaqueConstruction = Field(
        default=None,
        description='An OpaqueConstruction for walls with a Surface or '
            'Adiabatic boundary condition.'
    )

    exterior_construction: OpaqueConstruction = Field(
        default=None,
        description='An OpaqueConstruction for walls with an Outdoors '
            'boundary condition.'
    )

    ground_construction: OpaqueConstruction = Field(
        default=None,
        description='An OpaqueConstruction for walls with a Ground '
            'boundary condition.'
    )


class WallConstructionSet(_FaceSubSet):
    """A set of constructions for wall assemblies."""

    type: constr(regex='^WallConstructionSet$') = 'WallConstructionSet'


class FloorConstructionSet(_FaceSubSet):
    """A set of constructions for floor assemblies."""

    type: constr(regex='^FloorConstructionSet$') = 'FloorConstructionSet'


class RoofCeilingConstructionSet(_FaceSubSet):
    """A set of constructions for roof and ceiling assemblies."""

    type: constr(regex='^RoofCeilingConstructionSet$') = 'RoofCeilingConstructionSet'


class ApertureConstructionSetAbridged(NoExtraBaseModel):
    """A set of constructions for aperture assemblies."""

    type: constr(regex='^ApertureConstructionSetAbridged$') = \
        'ApertureConstructionSetAbridged'

    interior_construction: str = Field(
        default=None,
        min_length=1,
        max_length=100,
        description='Identifier for a WindowConstruction for apertures with an '
            'Outdoors boundary condition, False is_operable property, and a Wall '
            'face type for their parent face.'
    )

    window_construction: str = Field(
        default=None,
        min_length=1,
        max_length=100,
        description='Identifier for a WindowConstruction for all apertures with a '
            'Surface boundary condition.'
    )

    skylight_construction: str = Field(
        default=None,
        min_length=1,
        max_length=100,
        description='Identifier for a WindowConstruction for apertures with a Outdoors '
            'boundary condition, False is_operable property, and a RoofCeiling or '
            'Floor face type for their parent face.'
    )

    operable_construction: str = Field(
        default=None,
        min_length=1,
        max_length=100,
        description='Identifier for a WindowConstruction for all apertures with an '
            'Outdoors boundary condition and True is_operable property..'
    )


class ApertureConstructionSet(NoExtraBaseModel):
    """A set of constructions for aperture assemblies."""

    type: constr(regex='^ApertureConstructionSet$') = 'ApertureConstructionSet'

    interior_construction: WindowConstruction = Field(
        default=None,
        description='A WindowConstruction for apertures with an '
            'Outdoors boundary condition, False is_operable property, and a Wall '
            'face type for their parent face.'
    )

    window_construction: WindowConstruction = Field(
        default=None,
        description='A WindowConstruction for all apertures with a '
            'Surface boundary condition.'
    )

    skylight_construction: WindowConstruction = Field(
        default=None,
        description='A WindowConstruction for apertures with a Outdoors boundary '
            'condition, False is_operable property, and a RoofCeiling or '
            'Floor face type for their parent face.'
    )

    operable_construction: WindowConstruction = Field(
        default=None,
        description='A WindowConstruction for all apertures with an '
            'Outdoors boundary condition and True is_operable property..'
    )


class DoorConstructionSetAbridged(NoExtraBaseModel):
    """A set of constructions for door assemblies."""

    type: constr(regex='^DoorConstructionSetAbridged$') = 'DoorConstructionSetAbridged'

    interior_construction: str = Field(
        default=None,
        min_length=1,
        max_length=100,
        description='Identifier for an OpaqueConstruction for all opaque doors with a '
            'Surface boundary condition.'
    )

    exterior_construction: str = Field(
        default=None,
        min_length=1,
        max_length=100,
        description='Identifier for an OpaqueConstruction for opaque doors with an '
            'Outdoors boundary condition and a Wall face type for their parent face.'
    )

    overhead_construction: str = Field(
        default=None,
        min_length=1,
        max_length=100,
        description='Identifier for an OpaqueConstruction for opaque doors with an '
            'Outdoors boundary condition and a RoofCeiling or Floor type for '
            'their parent face.'
    )

    exterior_glass_construction: str = Field(
        default=None,
        min_length=1,
        max_length=100,
        description='Identifier for a WindowConstruction for all glass doors with an '
            'Outdoors boundary condition.'
    )

    interior_glass_construction: str = Field(
        default=None,
        min_length=1,
        max_length=100,
        description='Identifier for a WindowConstruction for all glass doors with a '
            'Surface boundary condition.'
    )


class DoorConstructionSet(NoExtraBaseModel):
    """A set of constructions for door assemblies."""

    type: constr(regex='^DoorConstructionSet$') = 'DoorConstructionSet'

    interior_construction: OpaqueConstruction = Field(
        default=None,
        description='An OpaqueConstruction for all opaque doors with a '
            'Surface boundary condition.'
    )

    exterior_construction: OpaqueConstruction = Field(
        default=None,
        description='An OpaqueConstruction for opaque doors with an '
            'Outdoors boundary condition and a Wall face type for their parent face.'
    )

    overhead_construction: OpaqueConstruction = Field(
        default=None,
        description='An OpaqueConstruction for opaque doors with an '
            'Outdoors boundary condition and a RoofCeiling or Floor type for '
            'their parent face.'
    )

    exterior_glass_construction: WindowConstruction = Field(
        default=None,
        description='A WindowConstruction for all glass doors with an '
            'Outdoors boundary condition.'
    )

    interior_glass_construction: WindowConstruction = Field(
        default=None,
        description='A WindowConstruction for all glass doors with a '
            'Surface boundary condition.'
    )


class ConstructionSetAbridged(IDdEnergyBaseModel):
    """A set of constructions for different surface types and boundary conditions."""

    type: constr(regex='^ConstructionSetAbridged$') = 'ConstructionSetAbridged'

    wall_set: WallConstructionSetAbridged = Field(
        default=None,
        description='A WallConstructionSetAbridged object for this ConstructionSet.'
    )

    floor_set: FloorConstructionSetAbridged = Field(
        default=None,
        description='A FloorConstructionSetAbridged object for this ConstructionSet.'
    )

    roof_ceiling_set: RoofCeilingConstructionSetAbridged = Field(
        default=None,
        description='A RoofCeilingConstructionSetAbridged object for this ConstructionSet.'
    )

    aperture_set: ApertureConstructionSetAbridged = Field(
        default=None,
        description='A ApertureConstructionSetAbridged object for this ConstructionSet.'
    )

    door_set: DoorConstructionSetAbridged = Field(
        default=None,
        description='A DoorConstructionSetAbridged object for this ConstructionSet.'
    )

    shade_construction: str = Field(
        default=None,
        min_length=1,
        max_length=100,
        description='The identifier of a ShadeConstruction to set the reflectance '
            'properties of all outdoor shades of all objects to which this '
            'ConstructionSet is assigned.'
    )

    air_boundary_construction: str = Field(
        default=None,
        min_length=1,
        max_length=100,
        description='The identifier of an AirBoundaryConstruction to set the properties '
            'of Faces with an AirBoundary type.'
    )


class ConstructionSet(ConstructionSetAbridged):
    """A set of constructions for different surface types and boundary conditions."""

    type: constr(regex='^ConstructionSet$') = 'ConstructionSet'

    wall_set: WallConstructionSet = Field(
        default=None,
        description='A WallConstructionSet object for this ConstructionSet.'
    )

    floor_set: FloorConstructionSet = Field(
        default=None,
        description='A FloorConstructionSet object for this ConstructionSet.'
    )

    roof_ceiling_set: RoofCeilingConstructionSet = Field(
        default=None,
        description='A RoofCeilingConstructionSet object for this ConstructionSet.'
    )

    aperture_set: ApertureConstructionSet = Field(
        default=None,
        description='A ApertureConstructionSet object for this ConstructionSet.'
    )

    door_set: DoorConstructionSet = Field(
        default=None,
        description='A DoorConstructionSet object for this ConstructionSet.'
    )

    shade_construction: ShadeConstruction = Field(
        default=None,
        description='A ShadeConstruction to set the reflectance '
            'properties of all outdoor shades of all objects to which this '
            'ConstructionSet is assigned.'
    )

    air_boundary_construction: AirBoundaryConstruction = Field(
        default=None,
        description='An AirBoundaryConstruction to set the properties '
            'of Faces with an AirBoundary type.'
    )


if __name__ == '__main__':
    print(ConstructionSetAbridged.schema_json(indent=2))
    print(ConstructionSet.schema_json(indent=2))



from honeybee_schema.model import Model
from honeybee_schema.energy.properties import ModelEnergyProperties
import os

# target folder where all of the samples live
root = os.path.dirname(os.path.dirname(__file__))
target_folder = os.path.join(root, 'samples', 'model')


def test_model_complete_multi_zone_office():
    file_path  = os.path.join(target_folder, 'model_complete_multi_zone_office.json')
    Model.parse_file(file_path)


def test_model_complete_multiroom_radiance():
    file_path  = os.path.join(target_folder, 'model_complete_multiroom_radiance.json')
    Model.parse_file(file_path) 


def test_model_complete_single_zone_office():
    file_path  = os.path.join(target_folder, 'model_complete_single_zone_office.json')
    Model.parse_file(file_path)


def test_model_complete_user_data():
    file_path  = os.path.join(target_folder, 'model_complete_user_data.json')
    Model.parse_file(file_path)


def test_model_complete_patient_room():
    file_path  = os.path.join(target_folder, 'model_complete_patient_room.json')
    Model.parse_file(file_path) 


def test_model_energy_shoe_box():
    file_path  = os.path.join(target_folder, 'model_energy_shoe_box.json')
    Model.parse_file(file_path)


def test_model_energy_detailed_loads():
    file_path  = os.path.join(target_folder, 'model_energy_detailed_loads.json')
    Model.parse_file(file_path) 


def test_model_energy_fixed_interval():
    file_path  = os.path.join(target_folder, 'model_energy_fixed_interval.json')
    Model.parse_file(file_path)


def test_model_energy_no_program():
    file_path  = os.path.join(target_folder, 'model_energy_no_program.json')
    Model.parse_file(file_path)


def test_model_energy_properties_office():
    file_path  = os.path.join(target_folder, 'model_energy_properties_office.json')
    ModelEnergyProperties.parse_file(file_path)



"""Base classs used by various schema objects."""
from pydantic import Field, validator
import datetime

from .._base import NoExtraBaseModel


class IDdEnergyBaseModel(NoExtraBaseModel):
    """Base class for all objects requiring a valid EnergyPlus identifier."""

    identifier: str = Field(
        ...,
        min_length=1,
        max_length=100,
        description='Text string for a unique object ID. This identifier remains '
            'constant as the object is mutated, copied, and serialized to different '
            'formats (eg. dict, idf, osm). This identifier is also used to reference '
            'the object across a Model. It must be < 100 characters, use only '
            'ASCII characters and exclude (, ; ! \\n \\t).'
    )

    @validator('identifier')
    def check_identifier(cls, v):
        assert all(ord(i) < 128 for i in v), 'Identifier contains non ASCII characters.'
        assert all(char not in v for char in (',', ';', '!', '\n', '\t')), \
            'Identifier contains an invalid character for EnergyPlus (, ; ! \\n \\t).'
        return v

    display_name: str = Field(
        default=None,
        description='Display name of the object with no character restrictions.'
    )


class DatedBaseModel(NoExtraBaseModel):
    """Base class for all objects needing to check for a valid Date."""

    @staticmethod
    def check_date(v, leap_pear_override=None):
        """Ensure valid date.
        
        Args:
            v: The Date array to be validated.
            leap_pear_override: Boolean to override the typical check for
                leap year serialization of date arrays. If not None, this
                boolean wil dictate whether the date is a leap year or not.
        """
        if (len(v) == 3 and v[2]) or leap_pear_override:
            try:
                datetime.date(2016, v[0] , v[1])
            except ValueError:
                raise ValueError('{}/{} is not a valid date.'.format(v[0], v[1]))
        else:
            try:
                datetime.date(2017, v[0] , v[1])
            except ValueError:
                raise ValueError('{}/{} is not a valid date.'.format(v[0], v[1]))
        return v



"""Model schema and the 5 geometry objects that define it."""
from pydantic import BaseModel, Field, validator, root_validator, constr, conlist
from typing import List, Union
from enum import Enum

from ._base import IDdBaseModel, NoExtraBaseModel
from .bc import Outdoors, Surface, Ground, Adiabatic
from .energy.properties import ShadeEnergyPropertiesAbridged, \
    DoorEnergyPropertiesAbridged, ApertureEnergyPropertiesAbridged, \
    FaceEnergyPropertiesAbridged, RoomEnergyPropertiesAbridged, \
    ModelEnergyProperties


class Plane(NoExtraBaseModel):

    type: constr(regex='^Plane$') = 'Plane'

    n: List[float] = Field(
        ...,
        description="Plane normal as 3 (x, y, z) values.",
        min_items=3,
        max_items=3
    )

    o: List[float] = Field(
        ...,
        description="Plane origin as 3 (x, y, z) values",
        min_items=3,
        max_items=3
    )

    x: List[float] = Field(
        default=None,
        description="Plane x-axis as 3 (x, y, z) values. If None, it is autocalculated.",
        min_items=3,
        max_items=3
    )


class Face3D(NoExtraBaseModel):
    """A single planar face in 3D space."""

    type: constr(regex='^Face3D$') = 'Face3D'

    boundary: List[conlist(float, min_items=3, max_items=3)] = Field(
        ...,
        min_items=3,
        description='A list of points representing the outer boundary vertices of '
            'the face. The list should include at least 3 points and each point '
            'should be a list of 3 (x, y, z) values.'
    )

    holes: List[conlist(conlist(float, min_items=3, max_items=3), min_items=3)] = Field(
        default=None,
        description='Optional list of lists with one list for each hole in the face.'
            'Each hole should be a list of at least 3 points and each point a list '
            'of 3 (x, y, z) values. If None, it will be assumed that there are no '
            'holes in the face.'
    )

    plane: Plane = Field(
        default=None,
        description='Optional Plane indicating the plane in which the face exists.'
            'If None, the plane will usually be derived from the boundary points.'
    )


class ShadePropertiesAbridged(BaseModel):

    type: constr(regex='^ShadePropertiesAbridged$') = 'ShadePropertiesAbridged'

    energy: ShadeEnergyPropertiesAbridged = Field(
        default=None
    )


class Shade(IDdBaseModel):

    type: constr(regex='^Shade$') = 'Shade'

    geometry: Face3D = Field(
        ...,
        description='Planar Face3D for the geometry.'
    )

    properties: ShadePropertiesAbridged = Field(
        ...,
        description='Extension properties for particular simulation engines '
            '(Radiance, EnergyPlus).'
    )


class DoorPropertiesAbridged(BaseModel):

    type: constr(regex='^DoorPropertiesAbridged$') = 'DoorPropertiesAbridged'

    energy: DoorEnergyPropertiesAbridged = Field(
        default=None
    )


class Door(IDdBaseModel):

    type: constr(regex='^Door$') = 'Door'

    geometry: Face3D = Field(
        ...,
        description='Planar Face3D for the geometry.'
    )

    boundary_condition: Union[Outdoors, Surface]

    @validator('boundary_condition')
    def suface_bc_objects(cls, v):
        if v.type == 'Surface':
            assert len(v.boundary_condition_objects) == 3, 'Door Surface boundary ' \
                'condition must have 3 boundary_condition_objects.'
        return v

    is_glass: bool = Field(
        False,
        description='Boolean to note whether this object is a glass door as opposed '
            'to an opaque door.'
    )

    indoor_shades: List[Shade] = Field(
        default=None,
        description='Shades assigned to the interior side of this object.'
    )

    outdoor_shades: List[Shade] = Field(
        default=None,
        description='Shades assigned to the exterior side of this object '
            '(eg. entryway awning).'
    )

    properties: DoorPropertiesAbridged = Field(
        ...,
        description='Extension properties for particular simulation engines '
            '(Radiance, EnergyPlus).'
    )

class AperturePropertiesAbridged(BaseModel):

    type: constr(regex='^AperturePropertiesAbridged$') = 'AperturePropertiesAbridged'

    energy: ApertureEnergyPropertiesAbridged = Field(
        default=None
    )


class Aperture(IDdBaseModel):

    type: constr(regex='^Aperture$') = 'Aperture'

    geometry: Face3D = Field(
        ...,
        description='Planar Face3D for the geometry.'
    )

    boundary_condition: Union[Outdoors, Surface]

    @validator('boundary_condition')
    def suface_bc_objects(cls, v):
        if v.type == 'Surface':
            assert len(v.boundary_condition_objects) == 3, 'Aperture Surface boundary ' \
                'condition must have 3 boundary_condition_objects.'
        return v

    is_operable: bool = Field(
        False,
        description='Boolean to note whether the Aperture can be opened for ventilation.'
    )

    indoor_shades: List[Shade] = Field(
        default=None,
        description='Shades assigned to the interior side of this object '
            '(eg. window sill, light shelf).'
    )

    outdoor_shades: List[Shade] = Field(
        default=None,
        description='Shades assigned to the exterior side of this object '
            '(eg. mullions, louvers).'
    )

    properties: AperturePropertiesAbridged = Field(
        ...,
        description='Extension properties for particular simulation engines '
            '(Radiance, EnergyPlus).'
    )


class FacePropertiesAbridged(BaseModel):

    type: constr(regex='^FacePropertiesAbridged$') = 'FacePropertiesAbridged'

    energy: FaceEnergyPropertiesAbridged = Field(
        default=None
    )


class FaceType(str, Enum):

    wall = 'Wall'
    floor = 'Floor'
    roof_ceiling = 'RoofCeiling'
    air_boundary = 'AirBoundary'


class Face(IDdBaseModel):

    type: constr(regex='^Face$') = 'Face'

    geometry: Face3D = Field(
        ...,
        description='Planar Face3D for the geometry.'
    )

    face_type: FaceType

    boundary_condition: Union[Ground, Outdoors, Adiabatic, Surface]

    @validator('boundary_condition')
    def suface_bc_objects(cls, v):
        if v.type == 'Surface':
            assert len(v.boundary_condition_objects) == 2, 'Face Surface boundary ' \
                'condition must have 2 boundary_condition_objects.'
        return v

    apertures: List[Aperture] = Field(
        default=None,
        description='Apertures assigned to this Face. Should be coplanar with this '
            'Face and completely within the boundary of the Face to be valid.'
    )

    doors: List[Door] = Field(
        default=None,
        description='Doors assigned to this Face. Should be coplanar with this '
            'Face and completely within the boundary of the Face to be valid.'
    )

    indoor_shades: List[Shade] = Field(
        default=None,
        description='Shades assigned to the interior side of this object.'
    )

    outdoor_shades: List[Shade] = Field(
        default=None,
        description='Shades assigned to the exterior side of this object '
            '(eg. balcony, overhang).'
    )

    properties: FacePropertiesAbridged = Field(
        ...,
        description='Extension properties for particular simulation engines '
            '(Radiance, EnergyPlus).'
    )

    @root_validator
    def chack_air_boundarys_are_interior(cls, values):
        """Check that all air wall faces have a Surface boundary condition."""
        face_type, bc = values.get('face_type'), values.get('boundary_condition')
        if face_type == 'AirBoundary':
            assert bc.type == 'Surface', \
                'AirBoundarys must have "Surface" boundary conditions.'
        return values


class RoomPropertiesAbridged(BaseModel):

    type: constr(regex='^RoomPropertiesAbridged$') = 'RoomPropertiesAbridged'

    energy: RoomEnergyPropertiesAbridged = Field(
        default=None
    )


class Room(IDdBaseModel):

    type: constr(regex='^Room$') = 'Room'

    faces: List[Face] = Field(
        ...,
        min_items=4,
        description='Faces that together form the closed volume of a room.'
    )

    indoor_shades: List[Shade] = Field(
        default=None,
        description='Shades assigned to the interior side of this object '
            '(eg. partitions, tables).'
    )

    outdoor_shades: List[Shade] = Field(
        default=None,
        description='Shades assigned to the exterior side of this object '
            '(eg. trees, landscaping).'
    )

    properties: RoomPropertiesAbridged = Field(
        ...,
        description='Extension properties for particular simulation engines '
            '(Radiance, EnergyPlus).'
    )

    multiplier: int = Field(
        1,
        ge=1,
        description='An integer noting how many times this Room is repeated. '
            'Multipliers are used to speed up the calculation when similar Rooms are '
            'repeated more than once. Essentially, a given simulation with the '
            'Room is run once and then the result is mutliplied by the multiplier.'
    )


class Units(str, Enum):
    meters = 'Meters'
    millimeters = 'Millimeters'
    feet = 'Feet'
    inches = 'Inches'
    centimeters = 'Centimeters'


class ModelProperties(BaseModel):

    type: constr(regex='^ModelProperties$') = 'ModelProperties'

    energy: ModelEnergyProperties = Field(
        default=None
    )


class Model(IDdBaseModel):

    type: constr(regex='^Model$') = 'Model'

    rooms: List[Room] = Field(
        default=None,
        description='A list of Rooms in the model.'
    )

    orphaned_faces: List[Face] = Field(
        default=None,
        description='A list of Faces in the model that lack a parent Room. Note that '
            'orphaned Faces are not acceptable for Models that are to be exported '
            'for energy simulation.'
    )

    orphaned_shades: List[Shade] = Field(
        default=None,
        description='A list of Shades in the model that lack a parent.'
    )

    orphaned_apertures: List[Aperture] = Field(
        default=None,
        description='A list of Apertures in the model that lack a parent Face. '
            'Note that orphaned Apertures are not acceptable for Models that are '
            'to be exported for energy simulation.'
    )

    orphaned_doors: List[Door] = Field(
        default=None,
        description='A list of Doors in the model that lack a parent Face. '
            'Note that orphaned Doors are not acceptable for Models that are '
            'to be exported for energy simulation.'
    )

    north_angle: float = Field(
        default=0,
        ge=0,
        lt=360,
        description='The clockwise north direction in degrees.'
    )

    units: Units = Field(
        default=Units.meters,
        description='Text indicating the units in which the model geometry exists. '
            'This is used to scale the geometry to the correct units for simulation '
            'engines like EnergyPlus, which requires all geometry be in meters.'
    )

    tolerance: float = Field(
        default=0,
        description='The maximum difference between x, y, and z values at which '
            'vertices are considered equivalent. This value should be in the Model '
            'units and it is used in a variety of checks, including checks for '
            'whether Room faces form a closed volume and subsequently correcting all '
            'face normals point outward from the Room. A value of 0 will result '
            'in no attempt to evaluate whether Room volumes are closed or check '
            'face direction. So it is recommended that this always be a positive '
            'number when such checks have not been performed on a Model. '
            'Typical tolerances for builing geometry range from 0.1 to 0.0001 '
            'depending on the units of the geometry.'
    )

    angle_tolerance: float = Field(
        default=0,
        description='The max angle difference in degrees that vertices are '
            'allowed to differ from one another in order to consider them colinear. '
            'This value is used in a variety of checks, including checks for '
            'whether Room faces form a closed volume and subsequently correcting all '
            'face normals point outward from the Room. A value of 0 will result '
            'in no attempt to evaluate whether the Room volumes is closed or check '
            'face direction. So it is recommended that this always be a positive '
            'number when such checks have not been performed on a given Model. '
            'Typical tolerances for builing geometry are often around 1 degree.'
    )

    properties: ModelProperties = Field(
        ...,
        description='Extension properties for particular simulation engines '
            '(Radiance, EnergyPlus).'
    )


if __name__ == '__main__':
    print(Model.schema_json(indent=2))



# coding=utf-8
from __future__ import division

from honeybee_energy.material.opaque import EnergyMaterialNoMass
from honeybee_energy.material.glazing import EnergyWindowMaterialSimpleGlazSys
from honeybee_energy.material.gas import EnergyWindowMaterialGasMixture, \
    EnergyWindowMaterialGasCustom
from honeybee_energy.material.shade import EnergyWindowMaterialShade, \
    EnergyWindowMaterialBlind

from  honeybee_energy.lib.materials import brick, concrete_lw, insulation, gypsum, \
    lowe_glass, clear_glass, air_gap

import os
import json


def material_opaque_brick(directory):
    dest_file = os.path.join(directory, 'material_opaque_brick.json')
    with open(dest_file, 'w') as fp:
        json.dump(brick.to_dict(), fp, indent=4)


def material_opaque_concrete(directory):
    dest_file = os.path.join(directory, 'material_opaque_concrete.json')
    with open(dest_file, 'w') as fp:
        json.dump(concrete_lw.to_dict(), fp, indent=4)


def material_opaque_insulation(directory):
    dest_file = os.path.join(directory, 'material_opaque_insulation.json')
    with open(dest_file, 'w') as fp:
        json.dump(insulation.to_dict(), fp, indent=4)


def material_opaque_gypsum(directory):
    dest_file = os.path.join(directory, 'material_opaque_gypsum.json')
    with open(dest_file, 'w') as fp:
        json.dump(gypsum.to_dict(), fp, indent=4)


def material_opaque_wall_gap(directory):
    wall_gap = EnergyMaterialNoMass('Generic Wall Air Gap', 0.15, roughness='Smooth')
    dest_file = os.path.join(directory, 'material_opaque_wall_gap.json')
    with open(dest_file, 'w') as fp:
        json.dump(wall_gap.to_dict(), fp, indent=4)


def material_window_glazing_clear(directory):
    dest_file = os.path.join(directory, 'material_window_glazing_clear.json')
    with open(dest_file, 'w') as fp:
        json.dump(clear_glass.to_dict(), fp, indent=4)


def material_window_glazing_lowe(directory):
    dest_file = os.path.join(directory, 'material_window_glazing_lowe.json')
    with open(dest_file, 'w') as fp:
        json.dump(lowe_glass.to_dict(), fp, indent=4)


def material_window_gas(directory):
    dest_file = os.path.join(directory, 'material_window_gas.json')
    with open(dest_file, 'w') as fp:
        json.dump(air_gap.to_dict(), fp, indent=4)


def material_window_gas_mixture(directory):
    argon_mix = EnergyWindowMaterialGasMixture(
        'Argon Air Mixture', 0.0125, ['Argon', 'Air'], [0.9, 0.1])
    dest_file = os.path.join(directory, 'material_window_gas_mixture.json')
    with open(dest_file, 'w') as fp:
        json.dump(argon_mix.to_dict(), fp, indent=4)


def material_window_gas_custom(directory):
    co2_gap = EnergyWindowMaterialGasCustom('CO2', 0.0125, 0.0146, 0.000014, 827.73)
    co2_gap.specific_heat_ratio = 1.4
    co2_gap.molecular_weight = 44
    dest_file = os.path.join(directory, 'material_window_gas_custom.json')
    with open(dest_file, 'w') as fp:
        json.dump(co2_gap.to_dict(), fp, indent=4)


def material_window_glazing_system(directory):
    simple_sys = EnergyWindowMaterialSimpleGlazSys(
        'Fixed Window 2.00 0.40 0.31', u_factor=1.98, shgc=0.4)
    dest_file = os.path.join(directory, 'material_window_glazing_system.json')
    with open(dest_file, 'w') as fp:
        json.dump(simple_sys.to_dict(), fp, indent=4)


def material_window_blind(directory):
    blinds = EnergyWindowMaterialBlind('Generic Aluminium Blinds')
    dest_file = os.path.join(directory, 'material_window_blind.json')
    with open(dest_file, 'w') as fp:
        json.dump(blinds.to_dict(), fp, indent=4)


def material_window_shade(directory):
    shades = EnergyWindowMaterialShade('White Diffusing Roller Shade')
    dest_file = os.path.join(directory, 'material_window_shade.json')
    with open(dest_file, 'w') as fp:
        json.dump(shades.to_dict(), fp, indent=4)


# run all functions within the file
master_dir = os.path.split(os.path.dirname(__file__))[0]
sample_directory = os.path.join(master_dir, 'samples', 'material')

material_opaque_brick(sample_directory)
material_opaque_concrete(sample_directory)
material_opaque_insulation(sample_directory)
material_opaque_gypsum(sample_directory)
material_opaque_wall_gap(sample_directory)
material_window_glazing_clear(sample_directory)
material_window_glazing_lowe(sample_directory)
material_window_gas(sample_directory)
material_window_gas_mixture(sample_directory)
material_window_gas_custom(sample_directory)
material_window_glazing_system(sample_directory)
material_window_blind(sample_directory)
material_window_shade(sample_directory)
